program gto(input,output);
{Program author: Steven S. Pietrobon
 20 Jun 2017 First release.
 27 Jan 2018 Updated for to include supersynchronous orbits.
 11 Apr 2018 Added option for any final orbit.
 18 Nov 2018 Added geosynchronous altitude output.
 20 Dec 2018 Minor corrections.
 21 Dec 2018 Corrected delta-V for ho >= 0.
 22 Dec 2018 Added burn altitudes.}

const R = 6378165; {m, Earth radius}
      mu = 3.986005e14; {m^3/s^2, Earth standard gravitational parameter}
      delta = 0.01*pi/180;
      T = 86164.09; {s, rotational period}

var hp1,ha1,rp1,ra1,theta1,theta2,v1,v2,dv1,v3,vo2,dv2,dvn,dvo,ho,ro:double;

function vp(rp,ra:double):double;
begin{perigee speed}
  vp := sqrt(2*mu/(rp*(rp/ra+1)));
end;{perigee speed}

function va(rp,ra:double):double;
begin{perigee speed}
  va := sqrt(2*mu/(ra*(ra/rp+1)));
end;{perigee speed}

function vo(ro:double):double;
begin{perigee speed}
  vo := sqrt(mu/ro);
end;{perigee speed}

function dv(v1,v2,theta:double):double;
begin{perigee speed}
  dv := sqrt(sqr(v1)+sqr(v2)-2*v1*v2*cos(theta));
end;{perigee speed}

procedure find_dv;
begin{find dv}
  if ha1 <= ho
    then begin{subsynchronous}
           v1 := vp(rp1,ra1);
           v2 := vp(rp1,ro);
           v3 := va(rp1,ro);
         end{subsynchronous}
    else begin{supersynchronous}
           v1 := va(rp1,ra1);
           v2 := va(ro,ra1);
           v3 := vp(ro,ra1);
         end;{supersynchronous}
  dv1 := dv(v1,v2,theta1);
  dv2 := dv(v3,vo2,theta2);
  dvn := dv1+dv2;
end;{find dv}

procedure write_dv;
begin{write dv}
  find_dv;
  writeln;
  if ha1 > ho
    then write('Burn at ',ha1/1000:7:1,' km: ')
    else write('Burn at ',hp1/1000:7:1,' km: ');
  writeln('theta1 = ',theta1*180/pi:5:2,' deg, dv1 = ',dv1:6:1,' m/s');
  write('Burn at ',ho/1000:7:1,' km: ');
  writeln('theta2 = ',theta2*180/pi:5:2,' deg, dv2 = ',dv2:6:1,' m/s');
  writeln('dv = ',dvn:6:1,' m/s');
end;{write dv}

begin{gto}
  writeln('Delta-V calculator by Steven S. Pietrobon. 22 Dec 2018.');
  writeln('Enter negative perigee height to exit program.');
  writeln('Enter negative final orbit for geosynchronous orbit.');
  writeln;
  repeat
    write('Enter initial perigee height (km): ');
    readln(hp1);
    if hp1 >= 0 then
      begin{valid hp1}
        hp1 := 1000*hp1;
        rp1 := R+hp1;
        write('Enter initial apogee height (km): ');
        readln(ha1);
        ha1 := 1000*ha1;
        ra1 := R+ha1;
        write('Enter required inclination change (deg): ');
        readln(theta2);
        write('Enter final orbit height (km): ');
        readln(ho);
        ho := 1000*ho;
        if ho < 0
          then begin{geosynchronous}
                 ro := exp(2*ln(sqrt(mu)*T/(2*pi))/3);
                 ho := ro-R;
                 writeln('Geosynchronous altitude = ',ho/1000:3:1,' km');
               end{geosynchronous}
          else ro := ho+R;
        vo2 := vo(ro);

        if ha1 <= ho
          then begin{subsynchronous}
                 theta1 := 0;
                 theta2 := theta2*pi/180;
                 write_dv;
                 repeat
                   dvo := dvn;
                   theta1 := theta1+delta;
                   theta2 := theta2-delta;
                   find_dv;
                 until (dvn > dvo) or (theta2 < 0);
                 theta1 := theta1-delta;
                 theta2 := theta2+delta;
                 write_dv;
               end{subsynchronous}
          else begin{supersynchronous}
                 theta1 := theta2*pi/180;
                 theta2 := 0;
                 write_dv;
                 repeat
                   dvo := dvn;
                   theta1 := theta1-delta;
                   theta2 := theta2+delta;
                   find_dv;
                 until (dvn > dvo) or (theta1 < 0);
                 theta1 := theta1+delta;
                 theta2 := theta2-delta;
                 write_dv;
               end;{supersynchronous}
        writeln;
      end;{valid perigee}
  until hp1 < 0;
end.{gto}